#!/usr/bin/ruby 
#
# Simple program to decode Galileo VICAR pictures.
# Convert all .lbl files to .jpg:   vicar .jpg *
# Convert foo.lbl to foo.raw:       vicar foo.lbl

def decode(filename,format)
    data = Hash.new
    current = data
    IO.foreach(filename) do |line|
        if line =~ /^\s*([\^A-Z_]+)\s*=\s*(.+)$/
            value = $2.strip
            if $1 == 'OBJECT' then
                current = Hash.new
                data[value] = current
            else current[$1] = value end
        elsif line =~ /^\s*END_OBJECT\s*$/
            current = data
        end
    end
    block_size = data['RECORD_BYTES'].to_i
    image = data['IMAGE']
    x_res = image['LINE_SAMPLES'].to_i
    y_res = image['LINES'].to_i
    prefix_bytes = image['LINE_PREFIX_BYTES'].to_i
    if data['^IMAGE'] =~ /\(\"([^\"]+)\"\,([0-9]+)\)/
        inname = $1.downcase
        basename = File.basename(inname,'.img')
        outname = basename + '.raw'
        input = File.new(inname)
        output = File.new(outname,'w')
        input.seek(block_size * ($2.to_i-1), IO::SEEK_SET)
        y_res.times do
            input.seek(prefix_bytes,IO::SEEK_CUR)
            output.write(input.read(x_res))
        end
        input.close
        output.close
    else puts "Can not load image data" end
    if format
        Kernel.system(  "convert -size #{x_res}x#{y_res} -depth 8 " +
                        "gray:#{outname} #{basename}#{format}")
        File.delete(outname)
    end
end

format = nil
ARGV.each do |arg|
    if arg =~ /^(\.\w+)$/
        format = $1
    else
        Dir[arg].each { |file| decode(file,format) if file =~ /\.lbl$/ }
    end
end

